# Do not modify this file by hand.
#
# It is automatically generated by src/api_wrappers/generatePythonAPIWrappers.py.
# (Run make api_wrappers to update it.)

import sys

from dxpy import DXHTTPRequest

def appAddCategories(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_add_categories()` instead.

    """
    print >> sys.stderr, "dxpy.appAddCategories is deprecated; please use app_add_categories instead."
    return app_add_categories(*args, **kwargs)

def app_add_categories(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/addCategories API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/addCategories
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/addCategories' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appAddDevelopers(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_add_developers()` instead.

    """
    print >> sys.stderr, "dxpy.appAddDevelopers is deprecated; please use app_add_developers instead."
    return app_add_developers(*args, **kwargs)

def app_add_developers(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/addDevelopers API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/addDevelopers
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/addDevelopers' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appAddTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_add_tags()` instead.

    """
    print >> sys.stderr, "dxpy.appAddTags is deprecated; please use app_add_tags instead."
    return app_add_tags(*args, **kwargs)

def app_add_tags(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/addTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/addTags
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/addTags' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appDelete(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_delete()` instead.

    """
    print >> sys.stderr, "dxpy.appDelete is deprecated; please use app_delete instead."
    return app_delete(*args, **kwargs)

def app_delete(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/delete API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/delete
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/delete' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_describe()` instead.

    """
    print >> sys.stderr, "dxpy.appDescribe is deprecated; please use app_describe instead."
    return app_describe(*args, **kwargs)

def app_describe(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/describe
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/describe' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appGet(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_get()` instead.

    """
    print >> sys.stderr, "dxpy.appGet is deprecated; please use app_get instead."
    return app_get(*args, **kwargs)

def app_get(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/get API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/get
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/get' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appInstall(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_install()` instead.

    """
    print >> sys.stderr, "dxpy.appInstall is deprecated; please use app_install instead."
    return app_install(*args, **kwargs)

def app_install(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/install API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/install
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/install' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appListCategories(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_list_categories()` instead.

    """
    print >> sys.stderr, "dxpy.appListCategories is deprecated; please use app_list_categories instead."
    return app_list_categories(*args, **kwargs)

def app_list_categories(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/listCategories API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/listCategories
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/listCategories' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appListDevelopers(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_list_developers()` instead.

    """
    print >> sys.stderr, "dxpy.appListDevelopers is deprecated; please use app_list_developers instead."
    return app_list_developers(*args, **kwargs)

def app_list_developers(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/listDevelopers API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/listDevelopers
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/listDevelopers' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appPublish(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_publish()` instead.

    """
    print >> sys.stderr, "dxpy.appPublish is deprecated; please use app_publish instead."
    return app_publish(*args, **kwargs)

def app_publish(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/publish API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/publish
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/publish' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appRemoveCategories(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_remove_categories()` instead.

    """
    print >> sys.stderr, "dxpy.appRemoveCategories is deprecated; please use app_remove_categories instead."
    return app_remove_categories(*args, **kwargs)

def app_remove_categories(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/removeCategories API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/removeCategories
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/removeCategories' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appRemoveDevelopers(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_remove_developers()` instead.

    """
    print >> sys.stderr, "dxpy.appRemoveDevelopers is deprecated; please use app_remove_developers instead."
    return app_remove_developers(*args, **kwargs)

def app_remove_developers(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/removeDevelopers API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/removeDevelopers
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/removeDevelopers' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appRemoveTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_remove_tags()` instead.

    """
    print >> sys.stderr, "dxpy.appRemoveTags is deprecated; please use app_remove_tags instead."
    return app_remove_tags(*args, **kwargs)

def app_remove_tags(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/removeTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/removeTags
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/removeTags' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appRun(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_run()` instead.

    """
    print >> sys.stderr, "dxpy.appRun is deprecated; please use app_run instead."
    return app_run(*args, **kwargs)

def app_run(app_name_or_id, alias=None, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /app-xxxx/run API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/run
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/run' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appUninstall(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_uninstall()` instead.

    """
    print >> sys.stderr, "dxpy.appUninstall is deprecated; please use app_uninstall instead."
    return app_uninstall(*args, **kwargs)

def app_uninstall(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/uninstall API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/uninstall
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/uninstall' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appUpdate(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_update()` instead.

    """
    print >> sys.stderr, "dxpy.appUpdate is deprecated; please use app_update instead."
    return app_update(*args, **kwargs)

def app_update(app_name_or_id, alias=None, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /app-xxxx/update API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app-xxxx%5B/yyyy%5D/update
    """
    fully_qualified_version = app_name_or_id + (('/' + alias) if alias else '')
    return DXHTTPRequest('/%s/update' % fully_qualified_version, input_params, always_retry=always_retry, **kwargs)

def appNew(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`app_new()` instead.

    """
    print >> sys.stderr, "dxpy.appNew is deprecated; please use app_new instead."
    return app_new(*args, **kwargs)

def app_new(input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /app/new API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app/new
    """
    return DXHTTPRequest('/app/new', input_params, always_retry=always_retry, **kwargs)

def appletAddTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_add_tags()` instead.

    """
    print >> sys.stderr, "dxpy.appletAddTags is deprecated; please use applet_add_tags instead."
    return applet_add_tags(*args, **kwargs)

def applet_add_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/addTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FaddTags
    """
    return DXHTTPRequest('/%s/addTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_describe()` instead.

    """
    print >> sys.stderr, "dxpy.appletDescribe is deprecated; please use applet_describe instead."
    return applet_describe(*args, **kwargs)

def applet_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fapplet-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletGet(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_get()` instead.

    """
    print >> sys.stderr, "dxpy.appletGet is deprecated; please use applet_get instead."
    return applet_get(*args, **kwargs)

def applet_get(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/get API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fapplet-xxxx%2Fget
    """
    return DXHTTPRequest('/%s/get' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletGetDetails(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_get_details()` instead.

    """
    print >> sys.stderr, "dxpy.appletGetDetails is deprecated; please use applet_get_details instead."
    return applet_get_details(*args, **kwargs)

def applet_get_details(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/getDetails API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#API-method%3A-%2Fclass-xxxx%2FgetDetails
    """
    return DXHTTPRequest('/%s/getDetails' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletListProjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_list_projects()` instead.

    """
    print >> sys.stderr, "dxpy.appletListProjects is deprecated; please use applet_list_projects instead."
    return applet_list_projects(*args, **kwargs)

def applet_list_projects(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/listProjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Cloning#API-method%3A-%2Fclass-xxxx%2FlistProjects
    """
    return DXHTTPRequest('/%s/listProjects' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletRemoveTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_remove_tags()` instead.

    """
    print >> sys.stderr, "dxpy.appletRemoveTags is deprecated; please use applet_remove_tags instead."
    return applet_remove_tags(*args, **kwargs)

def applet_remove_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/removeTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FremoveTags
    """
    return DXHTTPRequest('/%s/removeTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletRename(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_rename()` instead.

    """
    print >> sys.stderr, "dxpy.appletRename is deprecated; please use applet_rename instead."
    return applet_rename(*args, **kwargs)

def applet_rename(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/rename API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Name#API-method%3A-%2Fclass-xxxx%2Frename
    """
    return DXHTTPRequest('/%s/rename' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletRun(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_run()` instead.

    """
    print >> sys.stderr, "dxpy.appletRun is deprecated; please use applet_run instead."
    return applet_run(*args, **kwargs)

def applet_run(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /applet-xxxx/run API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fapplet-xxxx%2Frun
    """
    return DXHTTPRequest('/%s/run' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletSetProperties(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_set_properties()` instead.

    """
    print >> sys.stderr, "dxpy.appletSetProperties is deprecated; please use applet_set_properties instead."
    return applet_set_properties(*args, **kwargs)

def applet_set_properties(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /applet-xxxx/setProperties API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Properties#API-method%3A-%2Fclass-xxxx%2FsetProperties
    """
    return DXHTTPRequest('/%s/setProperties' % object_id, input_params, always_retry=always_retry, **kwargs)

def appletNew(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`applet_new()` instead.

    """
    print >> sys.stderr, "dxpy.appletNew is deprecated; please use applet_new instead."
    return applet_new(*args, **kwargs)

def applet_new(input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /applet/new API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fapplet%2Fnew
    """
    return DXHTTPRequest('/applet/new', input_params, always_retry=always_retry, **kwargs)

def containerClone(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_clone()` instead.

    """
    print >> sys.stderr, "dxpy.containerClone is deprecated; please use container_clone instead."
    return container_clone(*args, **kwargs)

def container_clone(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /container-xxxx/clone API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Cloning#API-method%3A-%2Fclass-xxxx%2Fclone
    """
    return DXHTTPRequest('/%s/clone' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_describe()` instead.

    """
    print >> sys.stderr, "dxpy.containerDescribe is deprecated; please use container_describe instead."
    return container_describe(*args, **kwargs)

def container_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /container-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Containers-for-Execution#API-method%3A-%2Fcontainer-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerDestroy(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_destroy()` instead.

    """
    print >> sys.stderr, "dxpy.containerDestroy is deprecated; please use container_destroy instead."
    return container_destroy(*args, **kwargs)

def container_destroy(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /container-xxxx/destroy API method.
    """
    return DXHTTPRequest('/%s/destroy' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerListFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_list_folder()` instead.

    """
    print >> sys.stderr, "dxpy.containerListFolder is deprecated; please use container_list_folder instead."
    return container_list_folder(*args, **kwargs)

def container_list_folder(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /container-xxxx/listFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FlistFolder
    """
    return DXHTTPRequest('/%s/listFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerMove(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_move()` instead.

    """
    print >> sys.stderr, "dxpy.containerMove is deprecated; please use container_move instead."
    return container_move(*args, **kwargs)

def container_move(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /container-xxxx/move API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2Fmove
    """
    return DXHTTPRequest('/%s/move' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerNewFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_new_folder()` instead.

    """
    print >> sys.stderr, "dxpy.containerNewFolder is deprecated; please use container_new_folder instead."
    return container_new_folder(*args, **kwargs)

def container_new_folder(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /container-xxxx/newFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FnewFolder
    """
    return DXHTTPRequest('/%s/newFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerRemoveFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_remove_folder()` instead.

    """
    print >> sys.stderr, "dxpy.containerRemoveFolder is deprecated; please use container_remove_folder instead."
    return container_remove_folder(*args, **kwargs)

def container_remove_folder(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /container-xxxx/removeFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FremoveFolder
    """
    return DXHTTPRequest('/%s/removeFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerRemoveObjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_remove_objects()` instead.

    """
    print >> sys.stderr, "dxpy.containerRemoveObjects is deprecated; please use container_remove_objects instead."
    return container_remove_objects(*args, **kwargs)

def container_remove_objects(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /container-xxxx/removeObjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FremoveObjects
    """
    return DXHTTPRequest('/%s/removeObjects' % object_id, input_params, always_retry=always_retry, **kwargs)

def containerRenameFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`container_rename_folder()` instead.

    """
    print >> sys.stderr, "dxpy.containerRenameFolder is deprecated; please use container_rename_folder instead."
    return container_rename_folder(*args, **kwargs)

def container_rename_folder(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /container-xxxx/renameFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FrenameFolder
    """
    return DXHTTPRequest('/%s/renameFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileAddTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_add_tags()` instead.

    """
    print >> sys.stderr, "dxpy.fileAddTags is deprecated; please use file_add_tags instead."
    return file_add_tags(*args, **kwargs)

def file_add_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/addTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FaddTags
    """
    return DXHTTPRequest('/%s/addTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileAddTypes(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_add_types()` instead.

    """
    print >> sys.stderr, "dxpy.fileAddTypes is deprecated; please use file_add_types instead."
    return file_add_types(*args, **kwargs)

def file_add_types(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/addTypes API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Types#API-method%3A-%2Fclass-xxxx%2FaddTypes
    """
    return DXHTTPRequest('/%s/addTypes' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileClose(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_close()` instead.

    """
    print >> sys.stderr, "dxpy.fileClose is deprecated; please use file_close instead."
    return file_close(*args, **kwargs)

def file_close(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/close API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Files#API-method%3A-%2Ffile-xxxx%2Fclose
    """
    return DXHTTPRequest('/%s/close' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_describe()` instead.

    """
    print >> sys.stderr, "dxpy.fileDescribe is deprecated; please use file_describe instead."
    return file_describe(*args, **kwargs)

def file_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Files#API-method%3A-%2Ffile-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileDownload(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_download()` instead.

    """
    print >> sys.stderr, "dxpy.fileDownload is deprecated; please use file_download instead."
    return file_download(*args, **kwargs)

def file_download(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/download API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Files#API-method%3A-%2Ffile-xxxx%2Fdownload
    """
    return DXHTTPRequest('/%s/download' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileGetDetails(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_get_details()` instead.

    """
    print >> sys.stderr, "dxpy.fileGetDetails is deprecated; please use file_get_details instead."
    return file_get_details(*args, **kwargs)

def file_get_details(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/getDetails API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#API-method%3A-%2Fclass-xxxx%2FgetDetails
    """
    return DXHTTPRequest('/%s/getDetails' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileListProjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_list_projects()` instead.

    """
    print >> sys.stderr, "dxpy.fileListProjects is deprecated; please use file_list_projects instead."
    return file_list_projects(*args, **kwargs)

def file_list_projects(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/listProjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Cloning#API-method%3A-%2Fclass-xxxx%2FlistProjects
    """
    return DXHTTPRequest('/%s/listProjects' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileRemoveTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_remove_tags()` instead.

    """
    print >> sys.stderr, "dxpy.fileRemoveTags is deprecated; please use file_remove_tags instead."
    return file_remove_tags(*args, **kwargs)

def file_remove_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/removeTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FremoveTags
    """
    return DXHTTPRequest('/%s/removeTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileRemoveTypes(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_remove_types()` instead.

    """
    print >> sys.stderr, "dxpy.fileRemoveTypes is deprecated; please use file_remove_types instead."
    return file_remove_types(*args, **kwargs)

def file_remove_types(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/removeTypes API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Types#API-method%3A-%2Fclass-xxxx%2FremoveTypes
    """
    return DXHTTPRequest('/%s/removeTypes' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileRename(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_rename()` instead.

    """
    print >> sys.stderr, "dxpy.fileRename is deprecated; please use file_rename instead."
    return file_rename(*args, **kwargs)

def file_rename(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/rename API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Name#API-method%3A-%2Fclass-xxxx%2Frename
    """
    return DXHTTPRequest('/%s/rename' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileSetDetails(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_set_details()` instead.

    """
    print >> sys.stderr, "dxpy.fileSetDetails is deprecated; please use file_set_details instead."
    return file_set_details(*args, **kwargs)

def file_set_details(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/setDetails API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#API-method%3A-%2Fclass-xxxx%2FsetDetails
    """
    return DXHTTPRequest('/%s/setDetails' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileSetProperties(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_set_properties()` instead.

    """
    print >> sys.stderr, "dxpy.fileSetProperties is deprecated; please use file_set_properties instead."
    return file_set_properties(*args, **kwargs)

def file_set_properties(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/setProperties API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Properties#API-method%3A-%2Fclass-xxxx%2FsetProperties
    """
    return DXHTTPRequest('/%s/setProperties' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileSetVisibility(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_set_visibility()` instead.

    """
    print >> sys.stderr, "dxpy.fileSetVisibility is deprecated; please use file_set_visibility instead."
    return file_set_visibility(*args, **kwargs)

def file_set_visibility(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/setVisibility API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Visibility#API-method%3A-%2Fclass-xxxx%2FsetVisibility
    """
    return DXHTTPRequest('/%s/setVisibility' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileUpload(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_upload()` instead.

    """
    print >> sys.stderr, "dxpy.fileUpload is deprecated; please use file_upload instead."
    return file_upload(*args, **kwargs)

def file_upload(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /file-xxxx/upload API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Files#API-method%3A-%2Ffile-xxxx%2Fupload
    """
    return DXHTTPRequest('/%s/upload' % object_id, input_params, always_retry=always_retry, **kwargs)

def fileNew(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`file_new()` instead.

    """
    print >> sys.stderr, "dxpy.fileNew is deprecated; please use file_new instead."
    return file_new(*args, **kwargs)

def file_new(input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /file/new API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Files#API-method%3A-%2Ffile%2Fnew
    """
    return DXHTTPRequest('/file/new', input_params, always_retry=always_retry, **kwargs)

def gtableAddRows(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_add_rows()` instead.

    """
    print >> sys.stderr, "dxpy.gtableAddRows is deprecated; please use gtable_add_rows instead."
    return gtable_add_rows(*args, **kwargs)

def gtable_add_rows(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/addRows API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/GenomicTables#API-method%3A-%2Fgtable-xxxx%2FaddRows
    """
    return DXHTTPRequest('/%s/addRows' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableAddTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_add_tags()` instead.

    """
    print >> sys.stderr, "dxpy.gtableAddTags is deprecated; please use gtable_add_tags instead."
    return gtable_add_tags(*args, **kwargs)

def gtable_add_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/addTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FaddTags
    """
    return DXHTTPRequest('/%s/addTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableAddTypes(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_add_types()` instead.

    """
    print >> sys.stderr, "dxpy.gtableAddTypes is deprecated; please use gtable_add_types instead."
    return gtable_add_types(*args, **kwargs)

def gtable_add_types(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/addTypes API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Types#API-method%3A-%2Fclass-xxxx%2FaddTypes
    """
    return DXHTTPRequest('/%s/addTypes' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableClose(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_close()` instead.

    """
    print >> sys.stderr, "dxpy.gtableClose is deprecated; please use gtable_close instead."
    return gtable_close(*args, **kwargs)

def gtable_close(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/close API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/GenomicTables#API-method%3A-%2Fgtable-xxxx%2Fclose
    """
    return DXHTTPRequest('/%s/close' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_describe()` instead.

    """
    print >> sys.stderr, "dxpy.gtableDescribe is deprecated; please use gtable_describe instead."
    return gtable_describe(*args, **kwargs)

def gtable_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/GenomicTables#API-method%3A-%2Fgtable-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableGet(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_get()` instead.

    """
    print >> sys.stderr, "dxpy.gtableGet is deprecated; please use gtable_get instead."
    return gtable_get(*args, **kwargs)

def gtable_get(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/get API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/GenomicTables#API-method%3A-%2Fgtable-xxxx%2Fget
    """
    return DXHTTPRequest('/%s/get' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableGetDetails(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_get_details()` instead.

    """
    print >> sys.stderr, "dxpy.gtableGetDetails is deprecated; please use gtable_get_details instead."
    return gtable_get_details(*args, **kwargs)

def gtable_get_details(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/getDetails API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#API-method%3A-%2Fclass-xxxx%2FgetDetails
    """
    return DXHTTPRequest('/%s/getDetails' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableListProjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_list_projects()` instead.

    """
    print >> sys.stderr, "dxpy.gtableListProjects is deprecated; please use gtable_list_projects instead."
    return gtable_list_projects(*args, **kwargs)

def gtable_list_projects(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/listProjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Cloning#API-method%3A-%2Fclass-xxxx%2FlistProjects
    """
    return DXHTTPRequest('/%s/listProjects' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableNextPart(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_next_part()` instead.

    """
    print >> sys.stderr, "dxpy.gtableNextPart is deprecated; please use gtable_next_part instead."
    return gtable_next_part(*args, **kwargs)

def gtable_next_part(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/nextPart API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/GenomicTables#API-method%3A-%2Fgtable-xxxx%2FnextPart
    """
    return DXHTTPRequest('/%s/nextPart' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableRemoveTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_remove_tags()` instead.

    """
    print >> sys.stderr, "dxpy.gtableRemoveTags is deprecated; please use gtable_remove_tags instead."
    return gtable_remove_tags(*args, **kwargs)

def gtable_remove_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/removeTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FremoveTags
    """
    return DXHTTPRequest('/%s/removeTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableRemoveTypes(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_remove_types()` instead.

    """
    print >> sys.stderr, "dxpy.gtableRemoveTypes is deprecated; please use gtable_remove_types instead."
    return gtable_remove_types(*args, **kwargs)

def gtable_remove_types(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/removeTypes API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Types#API-method%3A-%2Fclass-xxxx%2FremoveTypes
    """
    return DXHTTPRequest('/%s/removeTypes' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableRename(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_rename()` instead.

    """
    print >> sys.stderr, "dxpy.gtableRename is deprecated; please use gtable_rename instead."
    return gtable_rename(*args, **kwargs)

def gtable_rename(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/rename API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Name#API-method%3A-%2Fclass-xxxx%2Frename
    """
    return DXHTTPRequest('/%s/rename' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableSetDetails(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_set_details()` instead.

    """
    print >> sys.stderr, "dxpy.gtableSetDetails is deprecated; please use gtable_set_details instead."
    return gtable_set_details(*args, **kwargs)

def gtable_set_details(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/setDetails API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#API-method%3A-%2Fclass-xxxx%2FsetDetails
    """
    return DXHTTPRequest('/%s/setDetails' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableSetProperties(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_set_properties()` instead.

    """
    print >> sys.stderr, "dxpy.gtableSetProperties is deprecated; please use gtable_set_properties instead."
    return gtable_set_properties(*args, **kwargs)

def gtable_set_properties(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/setProperties API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Properties#API-method%3A-%2Fclass-xxxx%2FsetProperties
    """
    return DXHTTPRequest('/%s/setProperties' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableSetVisibility(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_set_visibility()` instead.

    """
    print >> sys.stderr, "dxpy.gtableSetVisibility is deprecated; please use gtable_set_visibility instead."
    return gtable_set_visibility(*args, **kwargs)

def gtable_set_visibility(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /gtable-xxxx/setVisibility API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Visibility#API-method%3A-%2Fclass-xxxx%2FsetVisibility
    """
    return DXHTTPRequest('/%s/setVisibility' % object_id, input_params, always_retry=always_retry, **kwargs)

def gtableNew(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`gtable_new()` instead.

    """
    print >> sys.stderr, "dxpy.gtableNew is deprecated; please use gtable_new instead."
    return gtable_new(*args, **kwargs)

def gtable_new(input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /gtable/new API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/GenomicTables#API-method%3A-%2Fgtable%2Fnew
    """
    return DXHTTPRequest('/gtable/new', input_params, always_retry=always_retry, **kwargs)

def jobDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`job_describe()` instead.

    """
    print >> sys.stderr, "dxpy.jobDescribe is deprecated; please use job_describe instead."
    return job_describe(*args, **kwargs)

def job_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /job-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fjob-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def jobStreamLog(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`job_stream_log()` instead.

    """
    print >> sys.stderr, "dxpy.jobStreamLog is deprecated; please use job_stream_log instead."
    return job_stream_log(*args, **kwargs)

def job_stream_log(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /job-xxxx/streamLog API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fjob-xxxx%2FstreamLog
    """
    return DXHTTPRequest('/%s/streamLog' % object_id, input_params, always_retry=always_retry, **kwargs)

def jobTerminate(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`job_terminate()` instead.

    """
    print >> sys.stderr, "dxpy.jobTerminate is deprecated; please use job_terminate instead."
    return job_terminate(*args, **kwargs)

def job_terminate(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /job-xxxx/terminate API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fjob-xxxx%2Fterminate
    """
    return DXHTTPRequest('/%s/terminate' % object_id, input_params, always_retry=always_retry, **kwargs)

def jobNew(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`job_new()` instead.

    """
    print >> sys.stderr, "dxpy.jobNew is deprecated; please use job_new instead."
    return job_new(*args, **kwargs)

def job_new(input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /job/new API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets and Entry Points#API-method%3A-%2Fjob%2Fnew
    """
    return DXHTTPRequest('/job/new', input_params, always_retry=always_retry, **kwargs)

def notificationsGet(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`notifications_get()` instead.

    """
    print >> sys.stderr, "dxpy.notificationsGet is deprecated; please use notifications_get instead."
    return notifications_get(*args, **kwargs)

def notifications_get(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /notifications/get API method.
    """
    return DXHTTPRequest('/notifications/get', input_params, always_retry=always_retry, **kwargs)

def notificationsMarkRead(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`notifications_mark_read()` instead.

    """
    print >> sys.stderr, "dxpy.notificationsMarkRead is deprecated; please use notifications_mark_read instead."
    return notifications_mark_read(*args, **kwargs)

def notifications_mark_read(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /notifications/markRead API method.
    """
    return DXHTTPRequest('/notifications/markRead', input_params, always_retry=always_retry, **kwargs)

def projectAddTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_add_tags()` instead.

    """
    print >> sys.stderr, "dxpy.projectAddTags is deprecated; please use project_add_tags instead."
    return project_add_tags(*args, **kwargs)

def project_add_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/addTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2FaddTags
    """
    return DXHTTPRequest('/%s/addTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectClone(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_clone()` instead.

    """
    print >> sys.stderr, "dxpy.projectClone is deprecated; please use project_clone instead."
    return project_clone(*args, **kwargs)

def project_clone(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /project-xxxx/clone API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Cloning#API-method%3A-%2Fclass-xxxx%2Fclone
    """
    return DXHTTPRequest('/%s/clone' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectDecreasePermissions(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_decrease_permissions()` instead.

    """
    print >> sys.stderr, "dxpy.projectDecreasePermissions is deprecated; please use project_decrease_permissions instead."
    return project_decrease_permissions(*args, **kwargs)

def project_decrease_permissions(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/decreasePermissions API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2FdecreasePermissions
    """
    return DXHTTPRequest('/%s/decreasePermissions' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_describe()` instead.

    """
    print >> sys.stderr, "dxpy.projectDescribe is deprecated; please use project_describe instead."
    return project_describe(*args, **kwargs)

def project_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectDestroy(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_destroy()` instead.

    """
    print >> sys.stderr, "dxpy.projectDestroy is deprecated; please use project_destroy instead."
    return project_destroy(*args, **kwargs)

def project_destroy(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/destroy API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2Fdestroy
    """
    return DXHTTPRequest('/%s/destroy' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectInvite(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_invite()` instead.

    """
    print >> sys.stderr, "dxpy.projectInvite is deprecated; please use project_invite instead."
    return project_invite(*args, **kwargs)

def project_invite(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /project-xxxx/invite API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Invitations and Join Requests#API-method%3A-%2Fproject-xxxx%2Finvite
    """
    return DXHTTPRequest('/%s/invite' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectLeave(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_leave()` instead.

    """
    print >> sys.stderr, "dxpy.projectLeave is deprecated; please use project_leave instead."
    return project_leave(*args, **kwargs)

def project_leave(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/leave API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2Fleave
    """
    return DXHTTPRequest('/%s/leave' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectListFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_list_folder()` instead.

    """
    print >> sys.stderr, "dxpy.projectListFolder is deprecated; please use project_list_folder instead."
    return project_list_folder(*args, **kwargs)

def project_list_folder(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/listFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FlistFolder
    """
    return DXHTTPRequest('/%s/listFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectMove(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_move()` instead.

    """
    print >> sys.stderr, "dxpy.projectMove is deprecated; please use project_move instead."
    return project_move(*args, **kwargs)

def project_move(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /project-xxxx/move API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2Fmove
    """
    return DXHTTPRequest('/%s/move' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectNewFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_new_folder()` instead.

    """
    print >> sys.stderr, "dxpy.projectNewFolder is deprecated; please use project_new_folder instead."
    return project_new_folder(*args, **kwargs)

def project_new_folder(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/newFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FnewFolder
    """
    return DXHTTPRequest('/%s/newFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectRemoveFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_remove_folder()` instead.

    """
    print >> sys.stderr, "dxpy.projectRemoveFolder is deprecated; please use project_remove_folder instead."
    return project_remove_folder(*args, **kwargs)

def project_remove_folder(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/removeFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FremoveFolder
    """
    return DXHTTPRequest('/%s/removeFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectRemoveObjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_remove_objects()` instead.

    """
    print >> sys.stderr, "dxpy.projectRemoveObjects is deprecated; please use project_remove_objects instead."
    return project_remove_objects(*args, **kwargs)

def project_remove_objects(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/removeObjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FremoveObjects
    """
    return DXHTTPRequest('/%s/removeObjects' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectRemoveTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_remove_tags()` instead.

    """
    print >> sys.stderr, "dxpy.projectRemoveTags is deprecated; please use project_remove_tags instead."
    return project_remove_tags(*args, **kwargs)

def project_remove_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/removeTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2FremoveTags
    """
    return DXHTTPRequest('/%s/removeTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectRenameFolder(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_rename_folder()` instead.

    """
    print >> sys.stderr, "dxpy.projectRenameFolder is deprecated; please use project_rename_folder instead."
    return project_rename_folder(*args, **kwargs)

def project_rename_folder(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /project-xxxx/renameFolder API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Folders-and-Deletion#API-method%3A-%2Fclass-xxxx%2FrenameFolder
    """
    return DXHTTPRequest('/%s/renameFolder' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectSetProperties(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_set_properties()` instead.

    """
    print >> sys.stderr, "dxpy.projectSetProperties is deprecated; please use project_set_properties instead."
    return project_set_properties(*args, **kwargs)

def project_set_properties(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/setProperties API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2FsetProperties
    """
    return DXHTTPRequest('/%s/setProperties' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectUpdate(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_update()` instead.

    """
    print >> sys.stderr, "dxpy.projectUpdate is deprecated; please use project_update instead."
    return project_update(*args, **kwargs)

def project_update(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /project-xxxx/update API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject-xxxx%2Fupdate
    """
    return DXHTTPRequest('/%s/update' % object_id, input_params, always_retry=always_retry, **kwargs)

def projectNew(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`project_new()` instead.

    """
    print >> sys.stderr, "dxpy.projectNew is deprecated; please use project_new instead."
    return project_new(*args, **kwargs)

def project_new(input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /project/new API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Projects#API-method%3A-%2Fproject%2Fnew
    """
    return DXHTTPRequest('/project/new', input_params, always_retry=always_retry, **kwargs)

def recordAddTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_add_tags()` instead.

    """
    print >> sys.stderr, "dxpy.recordAddTags is deprecated; please use record_add_tags instead."
    return record_add_tags(*args, **kwargs)

def record_add_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/addTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FaddTags
    """
    return DXHTTPRequest('/%s/addTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordAddTypes(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_add_types()` instead.

    """
    print >> sys.stderr, "dxpy.recordAddTypes is deprecated; please use record_add_types instead."
    return record_add_types(*args, **kwargs)

def record_add_types(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/addTypes API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Types#API-method%3A-%2Fclass-xxxx%2FaddTypes
    """
    return DXHTTPRequest('/%s/addTypes' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordClose(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_close()` instead.

    """
    print >> sys.stderr, "dxpy.recordClose is deprecated; please use record_close instead."
    return record_close(*args, **kwargs)

def record_close(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/close API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Data Object-Lifecycle#API-method%3A-%2Fclass-xxxx%2Fclose
    """
    return DXHTTPRequest('/%s/close' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_describe()` instead.

    """
    print >> sys.stderr, "dxpy.recordDescribe is deprecated; please use record_describe instead."
    return record_describe(*args, **kwargs)

def record_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Records#API-method%3A-%2Frecord-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordGetDetails(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_get_details()` instead.

    """
    print >> sys.stderr, "dxpy.recordGetDetails is deprecated; please use record_get_details instead."
    return record_get_details(*args, **kwargs)

def record_get_details(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/getDetails API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#API-method%3A-%2Fclass-xxxx%2FgetDetails
    """
    return DXHTTPRequest('/%s/getDetails' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordListProjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_list_projects()` instead.

    """
    print >> sys.stderr, "dxpy.recordListProjects is deprecated; please use record_list_projects instead."
    return record_list_projects(*args, **kwargs)

def record_list_projects(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/listProjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Cloning#API-method%3A-%2Fclass-xxxx%2FlistProjects
    """
    return DXHTTPRequest('/%s/listProjects' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordRemoveTags(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_remove_tags()` instead.

    """
    print >> sys.stderr, "dxpy.recordRemoveTags is deprecated; please use record_remove_tags instead."
    return record_remove_tags(*args, **kwargs)

def record_remove_tags(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/removeTags API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Tags#API-method%3A-%2Fclass-xxxx%2FremoveTags
    """
    return DXHTTPRequest('/%s/removeTags' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordRemoveTypes(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_remove_types()` instead.

    """
    print >> sys.stderr, "dxpy.recordRemoveTypes is deprecated; please use record_remove_types instead."
    return record_remove_types(*args, **kwargs)

def record_remove_types(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/removeTypes API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Types#API-method%3A-%2Fclass-xxxx%2FremoveTypes
    """
    return DXHTTPRequest('/%s/removeTypes' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordRename(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_rename()` instead.

    """
    print >> sys.stderr, "dxpy.recordRename is deprecated; please use record_rename instead."
    return record_rename(*args, **kwargs)

def record_rename(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/rename API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Name#API-method%3A-%2Fclass-xxxx%2Frename
    """
    return DXHTTPRequest('/%s/rename' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordSetDetails(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_set_details()` instead.

    """
    print >> sys.stderr, "dxpy.recordSetDetails is deprecated; please use record_set_details instead."
    return record_set_details(*args, **kwargs)

def record_set_details(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/setDetails API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#API-method%3A-%2Fclass-xxxx%2FsetDetails
    """
    return DXHTTPRequest('/%s/setDetails' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordSetProperties(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_set_properties()` instead.

    """
    print >> sys.stderr, "dxpy.recordSetProperties is deprecated; please use record_set_properties instead."
    return record_set_properties(*args, **kwargs)

def record_set_properties(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/setProperties API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Properties#API-method%3A-%2Fclass-xxxx%2FsetProperties
    """
    return DXHTTPRequest('/%s/setProperties' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordSetVisibility(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_set_visibility()` instead.

    """
    print >> sys.stderr, "dxpy.recordSetVisibility is deprecated; please use record_set_visibility instead."
    return record_set_visibility(*args, **kwargs)

def record_set_visibility(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /record-xxxx/setVisibility API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Visibility#API-method%3A-%2Fclass-xxxx%2FsetVisibility
    """
    return DXHTTPRequest('/%s/setVisibility' % object_id, input_params, always_retry=always_retry, **kwargs)

def recordNew(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`record_new()` instead.

    """
    print >> sys.stderr, "dxpy.recordNew is deprecated; please use record_new instead."
    return record_new(*args, **kwargs)

def record_new(input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /record/new API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Records#API-method%3A-%2Frecord%2Fnew
    """
    return DXHTTPRequest('/record/new', input_params, always_retry=always_retry, **kwargs)

def systemFindAffiliates(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_find_affiliates()` instead.

    """
    print >> sys.stderr, "dxpy.systemFindAffiliates is deprecated; please use system_find_affiliates instead."
    return system_find_affiliates(*args, **kwargs)

def system_find_affiliates(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/findAffiliates API method.
    """
    return DXHTTPRequest('/system/findAffiliates', input_params, always_retry=always_retry, **kwargs)

def systemFindApps(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_find_apps()` instead.

    """
    print >> sys.stderr, "dxpy.systemFindApps is deprecated; please use system_find_apps instead."
    return system_find_apps(*args, **kwargs)

def system_find_apps(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/findApps API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Search#API-method%3A-%2Fsystem%2FfindApps
    """
    return DXHTTPRequest('/system/findApps', input_params, always_retry=always_retry, **kwargs)

def systemFindDataObjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_find_data_objects()` instead.

    """
    print >> sys.stderr, "dxpy.systemFindDataObjects is deprecated; please use system_find_data_objects instead."
    return system_find_data_objects(*args, **kwargs)

def system_find_data_objects(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/findDataObjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Search#API-method%3A-%2Fsystem%2FfindDataObjects
    """
    return DXHTTPRequest('/system/findDataObjects', input_params, always_retry=always_retry, **kwargs)

def systemFindJobs(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_find_jobs()` instead.

    """
    print >> sys.stderr, "dxpy.systemFindJobs is deprecated; please use system_find_jobs instead."
    return system_find_jobs(*args, **kwargs)

def system_find_jobs(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/findJobs API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Search#API-method%3A-%2Fsystem%2FfindJobs
    """
    return DXHTTPRequest('/system/findJobs', input_params, always_retry=always_retry, **kwargs)

def systemFindProjects(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_find_projects()` instead.

    """
    print >> sys.stderr, "dxpy.systemFindProjects is deprecated; please use system_find_projects instead."
    return system_find_projects(*args, **kwargs)

def system_find_projects(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/findProjects API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Search#API-method%3A-%2Fsystem%2FfindProjects
    """
    return DXHTTPRequest('/system/findProjects', input_params, always_retry=always_retry, **kwargs)

def systemFindUsers(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_find_users()` instead.

    """
    print >> sys.stderr, "dxpy.systemFindUsers is deprecated; please use system_find_users instead."
    return system_find_users(*args, **kwargs)

def system_find_users(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/findUsers API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Search#API-method%3A-%2Fsystem%2FfindUsers
    """
    return DXHTTPRequest('/system/findUsers', input_params, always_retry=always_retry, **kwargs)

def systemFindProjectMembers(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_find_project_members()` instead.

    """
    print >> sys.stderr, "dxpy.systemFindProjectMembers is deprecated; please use system_find_project_members instead."
    return system_find_project_members(*args, **kwargs)

def system_find_project_members(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/findProjectMembers API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Search#API-method:-/system/findProjectMembers
    """
    return DXHTTPRequest('/system/findProjectMembers', input_params, always_retry=always_retry, **kwargs)

def systemGreet(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_greet()` instead.

    """
    print >> sys.stderr, "dxpy.systemGreet is deprecated; please use system_greet instead."
    return system_greet(*args, **kwargs)

def system_greet(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/greet API method.
    """
    return DXHTTPRequest('/system/greet', input_params, always_retry=always_retry, **kwargs)

def systemShortenURL(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`system_shorten_url()` instead.

    """
    print >> sys.stderr, "dxpy.systemShortenURL is deprecated; please use system_shorten_url instead."
    return system_shorten_url(*args, **kwargs)

def system_shorten_url(input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /system/shortenURL API method.
    """
    return DXHTTPRequest('/system/shortenURL', input_params, always_retry=always_retry, **kwargs)

def userDescribe(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`user_describe()` instead.

    """
    print >> sys.stderr, "dxpy.userDescribe is deprecated; please use user_describe instead."
    return user_describe(*args, **kwargs)

def user_describe(object_id, input_params={}, always_retry=True, **kwargs):
    """
    Invokes the /user-xxxx/describe API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Users#API-method%3A-%2Fuser-xxxx%2Fdescribe
    """
    return DXHTTPRequest('/%s/describe' % object_id, input_params, always_retry=always_retry, **kwargs)

def userUpdate(*args, **kwargs):
    """

    .. deprecated:: 0.42.0
       Use :func:`user_update()` instead.

    """
    print >> sys.stderr, "dxpy.userUpdate is deprecated; please use user_update instead."
    return user_update(*args, **kwargs)

def user_update(object_id, input_params={}, always_retry=False, **kwargs):
    """
    Invokes the /user-xxxx/update API method.

    For more info, see: https://wiki.dnanexus.com/API-Specification-v1.0.0/Users#API-method%3A-%2Fuser-xxxx%2Fupdate
    """
    return DXHTTPRequest('/%s/update' % object_id, input_params, always_retry=always_retry, **kwargs)




# Copyright (C) 2013 DNAnexus, Inc.
#
# This file is part of dx-toolkit (DNAnexus platform client libraries).
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may not
#   use this file except in compliance with the License. You may obtain a copy
#   of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

"""
DXGTable Handler
****************
"""

import os, sys, json, traceback
import cStringIO as StringIO
import concurrent.futures
from dxpy.bindings import *

DXGTABLE_HTTP_THREADS = 4

# Number of rows to request at a time when reading.
#
# TODO: adaptive buffer size. Start with small requests to improve interactivity and make
# progressively larger requests?
DEFAULT_TABLE_READ_ROW_BUFFER_SIZE = 40000

# Writing uses two buffers: one that contains the actual rows (list of Python lists) and the
# stringified data to send to the server (kept in a StringIO object). The row data is stringified
# when we have accumulated a fixed number of rows. The stringified data is sent to the server once
# its size exceeds a certain number of bytes.
#
# The row buffer should be large enough that we don't suffer a huge amount of overhead in
# stringifying, but the larger the row buffer, the more we could exceed the max byte size of the
# stringified buffer.
DEFAULT_TABLE_WRITE_ROW_BUFFER_SIZE = 10000
DEFAULT_TABLE_WRITE_REQUEST_SIZE = 1024*1024*64 # bytes

# Use this value for creating 'null' values in gtables.  Will be interpreted as null downstream.
# Available in apps as dxpy.NULL
NULL = - (1 << 31)

class DXGTable(DXDataObject):
    '''
    Remote GTable object handler.

    .. automethod:: _new
    '''

    _class = "gtable"

    _describe = staticmethod(dxpy.api.gtable_describe)
    _add_types = staticmethod(dxpy.api.gtable_add_types)
    _remove_types = staticmethod(dxpy.api.gtable_remove_types)
    _get_details = staticmethod(dxpy.api.gtable_get_details)
    _set_details = staticmethod(dxpy.api.gtable_set_details)
    _set_visibility = staticmethod(dxpy.api.gtable_set_visibility)
    _rename = staticmethod(dxpy.api.gtable_rename)
    _set_properties = staticmethod(dxpy.api.gtable_set_properties)
    _add_tags = staticmethod(dxpy.api.gtable_add_tags)
    _remove_tags = staticmethod(dxpy.api.gtable_remove_tags)
    _close = staticmethod(dxpy.api.gtable_close)
    _list_projects = staticmethod(dxpy.api.gtable_list_projects)

    _http_threadpool = None
    _http_threadpool_size = DXGTABLE_HTTP_THREADS

    @classmethod
    def set_http_threadpool_size(cls, num_threads):
        cls._http_threadpool_size = num_threads

    def __init__(self, dxid=None, project=None, mode=None, request_size=DEFAULT_TABLE_WRITE_REQUEST_SIZE):
        DXDataObject.__init__(self, dxid=dxid, project=project)
        if mode is None:
            self._close_on_exit = True
        else:
            if mode not in ['r', 'w', 'a']:
                raise ValueError("mode must be one of 'r', 'w', or 'a'")
            self._close_on_exit = (mode == 'w')

        self._write_request_size = request_size
        self._row_buf = []
        self._read_row_buffer_size = DEFAULT_TABLE_READ_ROW_BUFFER_SIZE
        self._write_row_buffer_size = DEFAULT_TABLE_WRITE_ROW_BUFFER_SIZE
        self._string_row_buf = None
        self._http_threadpool_futures = set()
        self._columns, self._col_names = None, None

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.flush()
        if self._close_on_exit:
            self.close()

    def __del__(self):
        '''
        Exceptions raised here in the destructor are IGNORED by Python! We will try and flush data
        here just as a safety measure, but you should not rely on this to flush your data! We will
        be really grumpy and complain if we detect unflushed data here.

        Use a context manager or flush the object explicitly to avoid this.

        In addition, when this is triggered by interpreter shutdown, the thread pool is not
        available, and we will wait for the request queue forever. In this case, we must revert to
        synchronous, in-thread flushing. We don't know how to detect this condition, so we'll use
        that for all destructor events.

        Neither this nor context managers are compatible with kwargs pass-through (so e.g. no
        custom auth).
        '''
        if len(self._row_buf) > 0 or (self._string_row_buf != None and self._string_row_buf.tell() > len('{"data": [')) or len(self._http_threadpool_futures) > 0:
            print >> sys.stderr, "=== WARNING! ==="
            print >> sys.stderr, "There is still unflushed data in the destructor of a DXGTable object!"
            print >> sys.stderr, "We will attempt to flush it now, but if an error were to occur, we could not report it back to you."
            print >> sys.stderr, "Your program could fail to flush the data but appear to succeed."
            print >> sys.stderr, "Instead, please call flush() or close(), or use the context managed version (e.g., with open_dxgtable(ID, mode='w') as gtable:)"
        try:
            self.flush(multithread=False)
        except Exception as e:
            print >> sys.stderr, "=== Exception occurred while flushing accumulated row data for %r" % (self._dxid,)
            traceback.print_exception(*sys.exc_info())
            raise

    def _check_row_is_valid(self, row):
        # TODO: if the user is using initFrom, we don't know what the schema looks like
        # (self._columns is None). In that case we can't do any local checks of the data's
        # validity.
        if self._columns is None:
            return
        if len(row) != len(self._columns):
            raise ValueError("Row has wrong number of columns (expected %d, got %d)" % (len(self._columns), len(row)))
        for index, (value, column) in enumerate(zip(row, self._columns)):
            if column['type'] == 'string':
                if not isinstance(value, basestring):
                    raise ValueError("Expected value in column %d to be a string, got %r instead" % (index, value))
            elif column['type'] == 'boolean':
                if value != True and value != False:
                    raise ValueError("Expected value in column %d to be a boolean, got %r instead" % (index, value))
            elif column['type'] == 'float' or column['type'] == 'double':
                if type(value) is not int and type(value) is not float:
                    raise ValueError("Expected value in column %d to be a number (int or float), got %r instead" % (index, value))
            elif column['type'].startswith('int') or column['type'].startswith('uint'):
                if type(value) is not int:
                    raise ValueError("Expected value in column %d to be an int, got %r instead" % (index, value))

    def _new(self, dx_hash, **kwargs):
        '''
        :param dx_hash: Standard hash populated in :func:`dxpy.bindings.DXDataObject.new()` containing attributes common to all data object classes.
        :type dx_hash: dict
        :param columns: An ordered list containing column descriptors. See :meth:`make_column_desc`.
        :type columns: list of column descriptors
        :param indices: An ordered list containing index descriptors. See :meth:`genomic_range_index()` and :meth:`lexicographic_index()`. If not provided, no indices are created.
        :type indices: list of index descriptors
        :param init_from: GTable from which to initialize the metadata including column and index specs
        :type init_from: :class:`DXGTable`

        Creates a new GTable with the columns described in *columns* and
        the indices described in *indices*.

        '''

        if "columns" in kwargs:
            if kwargs["columns"] is not None:
                dx_hash["columns"] = kwargs["columns"]
            del kwargs["columns"]
        else:
            if "init_from" not in kwargs:
                raise DXError("Column specs need to be specified if init_from is not used")

        if "indices" in kwargs:
            if kwargs["indices"] is not None:
                dx_hash["indices"] = kwargs["indices"]
            del kwargs["indices"]

        if "init_from" in kwargs:
            if kwargs["init_from"] is not None:
                if not isinstance(kwargs["init_from"], DXGTable):
                    raise DXError("Expected instance of DXGTable to init_from")
                dx_hash["initializeFrom"] = \
                    {"id": kwargs["init_from"].get_id(),
                     "project": kwargs["init_from"].get_proj_id()}
            del kwargs["init_from"]

        resp = dxpy.api.gtable_new(dx_hash, **kwargs)
        self.set_ids(resp["id"], dx_hash["project"])
        if "columns" in dx_hash:
            self._columns = dx_hash["columns"]

    def set_ids(self, dxid, project=None):
        '''
        :param dxid: Object ID
        :type dxid: string
        :param project: Project ID
        :type project: string

        Discards the currently stored ID and associates the handler with
        *dxid*. As a side effect, it also flushes the buffer for the
        previous GTable object if the buffer is nonempty.
        '''
        if self._dxid is not None:
            self.flush()

        DXDataObject.set_ids(self, dxid, project)

    def get_rows(self, query=None, columns=None, starting=None, limit=None, **kwargs):
        '''
        :param query: Query with which to filter the rows. See :meth:`genomic_range_query()` and :meth:`lexicographic_query()`.
        :type query: dict
        :param columns: List of column names to be included in the output. If not specified, each result contains the row ID followed by all column values. You can explicitly obtain the row ID by requesting the column ``__id__``.
        :type columns: list of strings
        :param starting: An optional row offset indicating the minimum row ID to return
        :type starting: integer
        :param limit: Maximum number of rows to be returned
        :type limit: integer
        :returns: A hash with the key-value pairs "length": the number of rows returned, "next": a value to use as "starting" in a subsequent query to get the next chunk of rows, and "data": a list of rows satisfying the query.

        Queries the GTable for rows matching the given query (or all
        rows, if no query is provided). If *columns* is set, the order
        of elements in the returned rows follows the ordering in
        *columns*. The *starting* and *limit* options restrict the row
        range of rows to return.

        Each row is returned as a list containing the row id followed by
        the values for each of the requested columns.

        .. note:: The
           :meth:`~dxpy.bindings.dxgtable.DXGTable.iterate_rows` and
           :meth:`~dxpy.bindings.dxgtable.DXGTable.iterate_query_rows`
           methods provide faster generator-based access to GTable rows.
           They transparently issue parallel background requests to
           obtain the data.

        Example::

            dxgtable = open_dxgtable("gtable-xxxx")
            query = genomic_range_query(chr="chromosome18", lo=30, hi=2049)
            for row in dxgtable.get_rows(query=query, limit=500):
                rowid = row[0]
                first_col_data = row[1]

        '''
        get_rows_params = {}
        if query is not None:
            get_rows_params['query'] = query
        if columns is not None:
            get_rows_params["columns"] = columns
        if starting is not None:
            get_rows_params["starting"] = starting
        if limit is not None:
            get_rows_params["limit"] = limit

        return dxpy.api.gtable_get(self._dxid, get_rows_params, always_retry=True, **kwargs)

    def get_columns(self, **kwargs):
        '''
        :returns: A list of column descriptors
        :rtype: list of dicts

        Returns a list of the descriptors for each of the GTable's
        columns.
        '''
        if self._columns is None:
            self._columns = self.describe(**kwargs).get("columns")
        return self._columns

    def get_col_names(self, **kwargs):
        '''
        :returns: A list of column names
        :rtype: list of strings

        Returns a list of the GTable's column names.
        '''
        if self._col_names is None:
            self._col_names = [col["name"] for col in self.get_columns(**kwargs)]
        return self._col_names

    def iterate_rows(self, start=0, end=None, columns=None, want_dict=False, **kwargs):
        """
        :param start: The row ID of the first row to return
        :type start: integer
        :param end: Return all rows before this row (return all rows until the end if None)
        :type end: integer or None
        :param columns: List of column names to be included in the output. If not specified, each result contains the row ID followed by all column values. You can explicitly obtain the row ID by requesting the column ``__id__``.
        :type columns: list of strings
        :param want_dict: If True, return a mapping of column names to values, instead of an array of values
        :type want_dict: boolean
        :rtype: generator

        Returns a generator that yields rows with IDs in the interval
        [*start*, *end*).

        """
        if want_dict:
            if columns is None:
                col_names = ['__id__'] + self.get_col_names(**kwargs)
            else:
                col_names = columns
        if self._http_threadpool is None:
            DXGTable._http_threadpool = dxpy.utils.get_futures_threadpool(max_workers=self._http_threadpool_size)

        request_iterator = self._generate_read_requests(start_row=start, end_row=end, columns=columns, **kwargs)

        for response in dxpy.utils.response_iterator(request_iterator, self._http_threadpool, max_active_tasks=self._http_threadpool_size):
            if want_dict:
                for row in response['data']:
                    yield dict(zip(col_names, row))
            else:
                for row in response['data']:
                    yield row

    def iterate_query_rows(self, query=None, columns=None, limit=None, want_dict=False, **kwargs):
        """
        :param query: Query with which to filter the rows. See :meth:`genomic_range_query()` and :meth:`lexicographic_query()`.
        :type query: dict
        :param columns: List of column names to be included in the output. If not specified, each result contains the row ID followed by all column values. You can explicitly obtain the row ID by requesting the column ``__id__``.
        :type columns: list of strings
        :param limit: Maximum number of rows to return (default is to return all matching rows)
        :type limit: int
        :param want_dict: If True, return a mapping of column names to values, instead of an array of values
        :type want_dict: boolean
        :rtype: generator

        Returns a generator that yields the rows of the table that match
        the given query parameters. If *query* is not given, all rows
        are returned in order of the row ID.

        Example::

            dxgtable = open_dxgtable(dxid)
            for row in dxgtable.iterate_query_rows(genomic_range_query(chr, lo, hi), [colname1, colname2]):
                print row

        """
        if want_dict:
            if columns is None:
                col_names = ['__id__'] + self.get_col_names(**kwargs)
            else:
                col_names = columns
        cursor = 0
        returned = 0
        while cursor is not None:
            if limit is not None and returned == limit:
                return
            resp = self.get_rows(query=query, columns=columns,
                                 starting=cursor,
                                 limit=(self._read_row_buffer_size if limit is None else min(limit - returned, self._read_row_buffer_size)),
                                 **kwargs)
            buffer = resp['data']
            cursor = resp['next']
            if len(buffer) < 1: break
            if want_dict:
                for row in buffer:
                    returned += 1
                    yield dict(zip(col_names, row))
            else:
                for row in buffer:
                    returned += 1
                    yield row

    def __iter__(self):
        return self.iterate_rows()

    # TODO: make this consume recarrays
    def add_rows(self, data, part=None, validate=True, **kwargs):
        '''
        :param data: List of rows to be added
        :type data: List of lists, list of mappings from column names to values (TODO), or mapping from column names to lists of values (TODO)
        :param part: The part ID to label the rows in data. Optional; it will be selected automatically if not given.
        :type part: integer
        :raises: :exc:`~dxpy.exceptions.DXGTableError`

        Adds the rows listed in data to the current GTable. If *part* is
        not given (recommended), rows may be queued up for addition
        internally and will be flushed to the remote server
        periodically.

        Example::

            with new_dxgtable([dxpy.DXGTable.make_column_desc("a", "string"),
                               dxpy.DXGTable.make_column_desc("b", "int32")], mode='w') as dxgtable:
                dxgtable.add_rows([["foo", 23], ["bar", 7]])

        '''

        if validate:
            for row in data:
                self._check_row_is_valid(row)
        if part is None:
            for row in data:
                self._row_buf.append(row)
                if len(self._row_buf) >= self._write_row_buffer_size:
                    self._flush_row_buf_to_string_buf()
                    if self._string_row_buf.tell() > self._write_request_size:
                        self._finalize_string_row_buf()
                        request_data = self._string_row_buf.getvalue()
                        self._string_row_buf = None
                        self._async_add_rows_request(self._dxid, request_data, jsonify_data=False, **kwargs)
                        del request_data
        else:
            dxpy.api.gtable_add_rows(self._dxid, {"data": data, "part": part}, **kwargs)

    def add_row(self, row, **kwargs):
        '''
        :param row: Row to be added
        :type data: List or mapping from column names to values (TODO)
        :raises: :exc:`~dxpy.exceptions.DXGTableError`

        Adds a single row to the current GTable. Rows may be queued up for addition internally
        and will be flushed to the remote server periodically.

        Example::

            with new_dxgtable([dxpy.DXGTable.make_column_desc("a", "string"),
                               dxpy.DXGTable.make_column_desc("b", "int32")], mode='w') as dxgtable:
                for i in range(1000):
                    dxgtable.add_row(["foo", i])
        '''
        self.add_rows([row], **kwargs)

    def get_unused_part_id(self, **kwargs):
        '''
        :returns: An unused part id
        :rtype: integer

        Obtains an unused part ID for use with
        :meth:`~dxpy.bindings.dxgtable.DXGTable.add_rows()`. Each call
        to this method on the same GTable (even from different clients)
        returns a different part ID.

        '''
        return dxpy.api.gtable_next_part(self._dxid, **kwargs)['part']

    def _flush_row_buf_to_string_buf(self):
        if self._string_row_buf == None:
            self._string_row_buf = StringIO.StringIO()
            self._string_row_buf.write('{"data": [')

        if len(self._row_buf) > 0:
            self._string_row_buf.write(json.dumps(self._row_buf)[1:])
            self._string_row_buf.seek(-1, os.SEEK_END) # chop off trailing "]"
            self._string_row_buf.write(", ")
            self._row_buf = []

    def _finalize_string_row_buf(self, part_id=None):
        if part_id == None:
            part_id = self.get_unused_part_id()

        # Temporary debug
        self._string_row_buf.seek(-2, os.SEEK_END) # chop off trailing ", "
        tail = self._string_row_buf.read()
        if tail != ', ':
            self._string_row_buf.seek(-100, os.SEEK_END)
            tail = self._string_row_buf.read()
            raise Exception("Unexpected buffer state: _finalize_string_row_buf called twice. Buffer tail: "+tail)
        
        self._string_row_buf.seek(-2, os.SEEK_END) # chop off trailing ", "
        self._string_row_buf.write('], "part": %s}' % str(part_id))

    def flush(self, multithread=True, **kwargs):
        '''
        Sends any rows in the internal buffer to the API server. If the buffer is empty, does nothing.
        '''
        if len(self._row_buf) > 0:
            self._flush_row_buf_to_string_buf()
        if self._string_row_buf != None and self._string_row_buf.tell() > len('{"data": ['):
            self._finalize_string_row_buf()
            request_data = self._string_row_buf.getvalue()
            self._string_row_buf = None
            if multithread:
                self._async_add_rows_request(self._dxid, request_data, jsonify_data=False, **kwargs)
            else:
                dxpy.api.gtable_add_rows(self._dxid, request_data, jsonify_data=False, **kwargs)

        if len(self._http_threadpool_futures) > 0:
            dxpy.utils.wait_for_all_futures(self._http_threadpool_futures)
            try:
                for future in self._http_threadpool_futures:
                    if future.exception() != None:
                        raise future.exception()
            finally:
                self._http_threadpool_futures = set()

    def close(self, block=False, **kwargs):
        '''
        :param block: If True, blocks until the remote GTable has closed
        :type block: boolean

        Closes the GTable.

        '''
        self.flush(**kwargs)

        dxpy.api.gtable_close(self._dxid, **kwargs)

        if block:
            self._wait_on_close(**kwargs)

    def wait_on_close(self, timeout=sys.maxint, **kwargs):
        '''
        :param timeout: Maximum amount of time to wait until the GTable is closed
        :type timeout: integer
        :raises: :exc:`~dxpy.exceptions.DXError` if the timeout is reached before the remote GTable has been closed

        Waits until the remote GTable is closed.
        '''
        self._wait_on_close(timeout, **kwargs)

    @staticmethod
    def make_column_desc(name, typename):
        """
        :param name: Column name
        :type name: string
        :param typename: Data type for the column
        :type typename: string
        :returns: A specially formatted dict that represents a column in the API
        :rtype: dict

        Returns a column descriptor with the given name and type. See
        the `API specification for GenomicTables
        <https://wiki.dnanexus.com/API-Specification-v1.0.0/GenomicTables#Columns>`_
        for information about the available types and acceptable column
        names.

        """

        return {"name": name, "type": typename}

    @staticmethod
    def genomic_range_index(chr, lo, hi, name="gri"):
        """
        :param chr: Name of the column containing chromosome names; must be a column of type string
        :type chr: string
        :param lo: Name of the column containing the low boundary of a genomic interval; must be a column of integral type
        :type lo: string
        :param hi: Name of the column containing the high boundary of a genomic interval; must be a column of integral type
        :type hi: string
        :param name: Name of the index
        :type name: string
        :returns: A specially formatted dict that represents an index schema in the API
        :rtype: dict

        Returns a genomic range index descriptor, for use with the
        :meth:`_new` method.

        """
        return {"name": name, "type": "genomic",
                "chr": chr, "lo": lo, "hi": hi}

    @staticmethod
    def lexicographic_index(columns, name):
        """
        :param columns: list of columns to be indexed, in order
        :type columns: list of hashes, each one of the form returned by :meth:`lexicographic_index_column()`
        :param name: Name of the index
        :type name: string
        :returns: A specially formatted dict that represents an index schema in the API
        :rtype: dict

        Returns a lexicographic index descriptor, for use with the
        :meth:`_new` method.

        """
        for column in columns:
            if type(column) is list:
                print >> sys.stderr, "Warning: passing a list of lists to lexicographic_index() is deprecated, please use lexicographic_index_column instead."

        return {"name": name, "type": "lexicographic", "columns": columns}

    @staticmethod
    def lexicographic_index_column(name, ascending=True, case_sensitive=None):
        """
        :param name: Name of the column to be indexed
        :type name: str
        :param ascending: Whether to order entries of this column in ascending order
        :type ascending: bool
        :param case_sensitive: if False, compare strings case-insensitively (default is True; only valid on string columns)
        :type case_sensitive: bool

        Returns a column descriptor for a lexicographic index, for use with the
        :meth:`lexicographic_index()` method.
        """
        result = {"name": name, "order": "asc" if ascending else "desc"}
        if case_sensitive is not None:
            result['caseSensitive'] = case_sensitive
        return result

    @staticmethod
    def genomic_range_query(chr, lo, hi, mode="overlap", index="gri"):
        """
        :param chr: Name of chromosome to be queried
        :type chr: string
        :param lo: Low boundary of query interval
        :type lo: integer
        :param hi: High boundary of query interval
        :type hi: integer
        :param mode: The type of query to perform ("overlap" or "enclose")
        :type mode: string
        :param index: Name of the genomic range index to use
        :type index: string
        :returns: A specially formatted dict that represents a GTable query in the API
        :rtype: dict

        Returns a query against a genomic range index of the table, for
        use with the :meth:`get_rows` or :meth:`iterate_query_rows`
        methods.

        """

        return {"index": index, "parameters": {"mode": mode,
                                               "coords": [chr, lo, hi] } }

    @staticmethod
    def lexicographic_query(query, index):
        """
        :param query: MongoDB-style query
        :type query: dict
        :param index: Name of the lexicographic index to use
        :type index: string
        :returns: A specially formatted dict that represents a GTable query in the API
        :rtype: dict

        Returns a query against a lexicographic index of the table, for
        use with the :meth:`get_rows` or :meth:`iterate_query_rows`
        methods.

        """

        return {"index": index, "parameters": query}

    def _async_add_rows_request(self, *args, **kwargs):
        kwargs['always_retry'] = True
        if self._http_threadpool is None:
            DXGTable._http_threadpool = dxpy.utils.get_futures_threadpool(max_workers=self._http_threadpool_size)

        while len(self._http_threadpool_futures) >= self._http_threadpool_size:
            future = dxpy.utils.wait_for_a_future(self._http_threadpool_futures)
            if future.exception() != None:
                raise future.exception()
            self._http_threadpool_futures.remove(future)
            del future

        future = self._http_threadpool.submit(dxpy.api.gtable_add_rows, *args, **kwargs)
        self._http_threadpool_futures.add(future)

    def _generate_read_requests(self, start_row=0, end_row=None, query=None, columns=None, **kwargs):
        if end_row is None:
            end_row = int(self.describe(**kwargs)['length'])
        kwargs['query'] = query
        kwargs['columns'] = columns
        cursor = start_row
        while cursor < end_row:
            request_size = min(self._read_row_buffer_size, end_row - cursor)
            my_kwargs = dict(kwargs)
            my_kwargs['starting'] = cursor
            my_kwargs['limit'] = request_size
            yield self.get_rows, [], my_kwargs
            cursor += request_size



#!/usr/bin/env python
#
# Copyright (C) 2013 DNAnexus, Inc.
#
# This file is part of dx-toolkit (DNAnexus platform client libraries).
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may not
#   use this file except in compliance with the License. You may obtain a copy
#   of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import dxpy
import argparse
import sys

parser = argparse.ArgumentParser(description="Export Genes object to GFF File")
parser.add_argument("genes_id", help="Genes table id to read from")
parser.add_argument("--output", dest="file_name", default=None, help="Name of file to write GFF to.  If not given GFF file will be printed to stdout.")
parser.add_argument("--only_genes_types", action="store_true", default=False, dest="only_genes_types", help="Name of file to write GFF to.  If not given GFF file will be printed to stdout.")


def main(**kwargs):

    if len(kwargs) == 0:
        opts = parser.parse_args(sys.argv[1:])
    else:
        opts = parser.parse_args(kwargs)

    if opts.genes_id == None:
        parser.print_help()
        sys.exit(1)
        
    if opts.file_name != None:
        outputFile = open(opts.file_name, 'w')
    else:
        outputFile = None
        

    if opts.genes_id == None:
        parser.print_help()
        sys.exit(1)

    tableId = opts.genes_id
    table = dxpy.DXGTable(tableId)
    
    genesTypes = {"exon": True, "CDS":True, "5' UTR": True, "3' UTR": True, "transcript":True, "gene":True}
    translatedTypes = {"transcript":"mRNA", "5' UTR": "five_prime_UTR", "3' UTR":"three_prime_UTR"}
    
    columns = table.get_col_names()
    idColumn = None
    parentColumn = None
    if "ID" in columns:
        idColumn = "ID"
    elif "Id" in columns:
        idColumn = "Id"
    elif "id" in columns:
        idColumn = "id"
    else:
        idColumn = "span_id"
        
    if "Parent" in columns:
        parentColumn = "Parent"
    elif "PARENT" in columns:
        parentColumn = "PARENT"
    elif "parent" in columns:
        parentColumn = "parent"
    else:
        parentColumn = "parent_id"
    
    for row in table.iterate_rows(want_dict=True):
        typ = row["type"]
        if opts.only_genes_types == False or genesTypes.get(typ) != None:
            if translatedTypes.get(typ) != None:
                typ = translatedTypes[typ]

            reservedColumns = ["chr", "lo", "hi", "span_id", "type", "strand", "score", "is_coding", "parent_id", "frame", "source", "__id__", "ID", "Id", "id", "Parent", "PARENT", "parent"]
            attributes = ""
            
            rowId = str(row[idColumn])
            parentId = str(row[parentColumn])
        
            attributes += "ID=\"" + rowId + "\";" 
            if not (parentColumn == "parent_id" and parentId == "-1"):
                attributes += "Parent=\"" + parentId + "\";"
            
            for k, v in row.iteritems():
                if k not in reservedColumns and v != '':
                    attributes += k + "=" + '"'+str(v)+'";'

            chromosome = row["chr"]
            lo = str(row["lo"] + 1)
            hi = str(row["hi"])
    
            strand = row["strand"]
            if strand == '':
                strand = '.'
            if row["frame"] == -1:
                frame = '.'
            else:
                frame = str(row["frame"])
            source = '.'
            
            # 2**31 and 2**31-1 are legacy null values that will be removed when possible
            if row.get("score") == None:
                score = "."
            if row["score"] == dxpy.NULL or row["score"] == 2**31-1 or row["score"] == float(2**31):
                score = "."
            else:
                score = str(row["score"])
            
            if row.get("source") != None:
                if row["source"] !=  '':
                    source = row["source"]
            result = "\t".join([chromosome, source, typ, lo, hi, score, strand, frame, attributes.rstrip(";")])+"\n"
            if outputFile != None:
                outputFile.write(result)
            else:
                sys.stdout.write(result)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
#
# Copyright (C) 2013 DNAnexus, Inc.
#
# This file is part of gtf_importer.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may not
#   use this file except in compliance with the License. You may obtain a copy
#   of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import dxpy
import string
import random
import sys

import magic
import subprocess
import argparse

parser = argparse.ArgumentParser(description='Import a local GTF file as a Spans or Genes object.')
parser.add_argument('fileName', help='local fileName to import')
parser.add_argument('reference', help='ID of ContigSet object (reference) that this GTF file annotates')
parser.add_argument('--outputName', dest='outputName', default='', help='what to name the output. if none provided, the name of the input file will be used with gtf file extension removed.')
parser.add_argument('--file_id', default=None, help='the DNAnexus file-id of the original file. If provided, a link to this id will be added in the type details')
parser.add_argument('--additional_type', default=[], action='append', help='This will be added to the list of object types (in addition to the type \"Genes\" which is added automatically')
parser.add_argument('--property_key', default=[], action='append', help='The keys in key-value pairs that will be added to the details of the object. The nth property key will be paired with the nth property value. The number of keys must equal the number of values provided')
parser.add_argument('--property_value', default=[], action='append', help='The values in key-value pairs that will be added to the details of the object. The nth property key will be paired with the nth property value. The number of keys must equal the number of values provided')
parser.add_argument('--tag', default=[], action='append', help='"A set of tags (string labels) that will be added to the resulting Variants table object. (You can use tags and properties to better describe and organize your data)')

@dxpy.entry_point('main')
def importGTF(**args):

    if len(args) == 0:
        command_line_args = parser.parse_args(sys.argv[1:])
        fileName = command_line_args.fileName
        reference = command_line_args.reference
        outputName = command_line_args.outputName
        tag = command_line_args.tag
        property_key = command_line_args.property_key
        property_value = command_line_args.property_value
        additional_type = command_line_args.additional_type
        file_id = command_line_args.file_id
    else:
        fileName = args['fileName']
        reference = args['reference']
        outputName = ''
        if args.get('outputName') != None:
            outputName = args['outputName']
        tag = []
        if args.get('tag'):
            tag = args['tag']
        if args.get('property_key') != None:
            property_key = args['property_key']
        if args.get('property_value') != None:
            property_value = args['property_value']
        if args.get('additional_type') != None:
            additional_type = args['additional_type']
        if args.get('file_id') != None:
            file_id = args['file_id']

    inputFileName = unpack(fileName)

    capturedTypes = {"5UTR": "5' UTR", "3UTR": "3' UTR", "CDS": "CDS", "inter": "intergenic", "inter_CNS": "intergenic_conserved", "intron_CNS": "intron_conserved", "exon": "exon", "transcript": "transcript", "gene":"gene", "stop_codon": "stop_codon", "start_codon":"start_codon"}
    
    #Rows of this type will not be written to the gtable as their information is fully encompassed by the rest of the data

    ##Isolate the attribute tags from the file and check integrity
    spansTable, additionalColumns = constructTable(inputFileName)
    spansTable.add_tags(tag)

    types = ["Genes", "gri"]
    for x in additional_type:
        types.append(x)
    spansTable.add_types(types)
    details = {'original_contigset': dxpy.dxlink(reference)}

    if len(property_key) != len(property_value):
        raise dxpy.AppError("Expected each provided property to have a corresponding value")
    for i in range(len(property_key)):
        details[property_key[i]] = property_value[i]
    for x in additional_type:
        types.append(x)

    if file_id != None:
        details['original_file'] = dxpy.dxlink(file_id)
    spansTable.set_details(details)
    if outputName == '':
        spansTable.rename(fileName)
    else:
        spansTable.rename(outputName)

    #This passes through the file calculates the gene and transcript models 
    genes = {}
    transcripts = {}
    spanId = 0
    frames = {}
    stopCodons = {}
    

    inputFile = open(inputFileName, 'r')
    for line in inputFile:
        if line[0] != "#":
            values = parseLine(line, capturedTypes)

            if values["type"] == "CDS":
                if frames.get(values["transcriptId"]) == None:
                    frames[values["transcriptId"]] = {}
                frames[values["transcriptId"]][values["lo"]] = values["frame"]
                
            for [element, hashId, elementType] in [[genes, values["geneId"], "geneName"], [transcripts, values["transcriptId"], "transcriptName"]]:   

                if element.get(hashId) == None:
                    element[hashId] = {values["chromosome"]: {"lo":values["lo"], "hi":values["hi"], "codingLo": -1, "codingHi": -1, "strand":values["strand"], "score":values["score"], "geneId":values["geneId"], "coding":False, "spanId": spanId, "name":values[elementType], "originalGeneId": values["attributes"]["gene_id"], "originalTranscriptId": values["attributes"]["transcript_id"]}}
                    spanId += 1
                elif element[hashId].get(values["chromosome"]) == None:
                    element[hashId][values["chromosome"]] = {"lo":values["lo"], "hi":values["hi"], "codingLo": -1, "codingHi": -1, "strand":values["strand"], "score":values["score"], "geneId":values["geneId"], "coding":False, "spanId": spanId, "name":values[elementType], "originalGeneId": values["attributes"]["gene_id"], "originalTranscriptId": values["attributes"]["transcript_id"]}
                    spanId += 1
                else:
                    if values["lo"] < element[hashId][values["chromosome"]]["lo"]:
                        element[hashId][values["chromosome"]]["lo"] = values["lo"]
                    if values["hi"] > element[hashId][values["chromosome"]]["hi"]:
                        element[hashId][values["chromosome"]]["hi"] = values["hi"]

            if values["type"] == "stop_codon":
                if stopCodons.get(values["transcriptId"]) == None:
                    stopCodons[values["transcriptId"]] = [[values["lo"], values["hi"]]]
                else:
                    stopCodons[values["transcriptId"]].append([values["lo"], values["hi"]])

            if values["type"] == "CDS" or values["type"] == "start_codon" or values["type"] == "stop_codon":
                if values["hi"] > transcripts[values["transcriptId"]][values["chromosome"]]["codingHi"]:
                    transcripts[values["transcriptId"]][values["chromosome"]]["codingHi"] = values["hi"]
                if values["lo"] < transcripts[values["transcriptId"]][values["chromosome"]]["codingLo"] or transcripts[values["transcriptId"]][values["chromosome"]]["codingLo"] == -1:
                    transcripts[values["transcriptId"]][values["chromosome"]]["codingLo"] = values["lo"]
                genes[values["geneId"]][values["chromosome"]]["coding"] = True
                transcripts[values["transcriptId"]][values["chromosome"]]["coding"] = True

    for gId, chrList in genes.iteritems():
        for k, v in chrList.iteritems():
            entry = [k, v["lo"], v["hi"], v["name"], v["spanId"], "gene", v["strand"], v["score"], v["coding"], -1, -1, '', '', v["originalGeneId"], '']
            for x in additionalColumns:
                if x != "gene_id" and x != "transcript_id":
                    entry.append('')
            spansTable.add_rows([entry])
    for gId, chrList in transcripts.iteritems():
        for k, v in chrList.iteritems():
            entry = [k, v["lo"], v["hi"], v["name"], v["spanId"], "transcript", v["strand"], v["score"], genes[v["geneId"]][k]["coding"], genes[v["geneId"]][k]["spanId"], -1, '', '', v["originalGeneId"], v["originalTranscriptId"]]
            for x in additionalColumns:
                if x != "gene_id" and x != "transcript_id":
                    entry.append('')
            spansTable.add_rows([entry])

    exons = {}
    inputFile = open(inputFileName, 'r')
    
    for line in inputFile:
        if line[0] != "#":
            values = parseLine(line, capturedTypes)

            if exons.get(values["transcriptId"]) != None:
                if exons[values["transcriptId"]].get(values["chromosome"]) == None:
                    exons[values["transcriptId"]][values["chromosome"]] = []
            else:
                exons[values["transcriptId"]] = {values["chromosome"] : []}

            if capturedTypes.get(values["type"]) != None:
                #If type is 5'UTR, 3'UTR, intergenic, or conserved intron, type is always noncoding
                if values["type"] == "5UTR" or values["type"] == "3UTR" or values["type"] == "inter" or values["type"] == "inter_CNS" or values["type"] == "intron_CNS":
                    writeEntry(spansTable, spanId, exons[values["transcriptId"]], additionalColumns, values["chromosome"], values["lo"], values["hi"], values["attributes"], [values["chromosome"], values["lo"], values["hi"], values["name"], spanId, capturedTypes[values["type"]], values["strand"], values["score"], False, transcripts[values["transcriptId"]]["spanId"], values["frame"], '', values["source"]])

                if "exon_number" in values["attributes"]:
                    values["transcriptName"] += "." + values["attributes"]["exon_number"]

                #If type is CDS, always of type coding
                if values["type"] == "CDS":
                    if stopCodons.get(values["transcriptId"]) != None:
                        for x in stopCodons[values["transcriptId"]]:
                            if values["hi"] == x[0]:
                                values["hi"] = x[1]
                                break                            
                    if [values["lo"], values["hi"]] not in exons[values["transcriptId"]][values["chromosome"]]:
                        spanId = writeEntry(spansTable, spanId, exons[values["transcriptId"]], additionalColumns, values["chromosome"], values["lo"], values["hi"], values["attributes"], [values["chromosome"], values["lo"], values["hi"], values["transcriptName"], spanId, capturedTypes[values["type"]], values["strand"], values["score"], True, transcripts[values["transcriptId"]][values["chromosome"]]["spanId"], values["frame"], '', values["source"]])

                #If type is exon do calculation as to whether coding or non-coding
                if values["type"] == "stop_codon":
                    values["type"] = "exon"
                    values["frame"] = 3 - (values["hi"] - values["lo"])
                    #if values["strand"] == "-":
                    #    values["lo"] = transcripts[values["transcriptId"]][values["chromosome"]]["lo"]
                    #else:
                    #    values["hi"] = transcripts[values["transcriptId"]][values["chromosome"]]["hi"]
                    
                if values["type"] == "exon":
                    if (transcripts[values["transcriptId"]][values["chromosome"]]["codingLo"] != -1 and transcripts[values["transcriptId"]][values["chromosome"]]["codingHi"] != -1):
                        if frames.get(values["transcriptId"]) != None:
                            if frames[values["transcriptId"]].get(values["lo"]) != None:
                                values["frame"] = frames[values["transcriptId"]][values["lo"]]

                        for x in splitExons(transcripts[values["transcriptId"]], values["chromosome"], values["lo"], values["hi"], values["strand"]):
                            spanId = writeEntry(spansTable, spanId, exons[values["transcriptId"]], additionalColumns, values["chromosome"], x[1], x[2], values["attributes"], [values["chromosome"], x[1], x[2], values["transcriptName"], spanId, x[0], values["strand"], values["score"], x[3], transcripts[values["transcriptId"]][values["chromosome"]]["spanId"], values["frame"], '', values["source"]])
                    else:
                        spanId = writeEntry(spansTable, spanId, exons[values["transcriptId"]], additionalColumns, values["chromosome"], values["lo"], values["hi"], values["attributes"],  [values["chromosome"], values["lo"], values["hi"], values["transcriptName"], spanId, capturedTypes[values["type"]], values["strand"], values["score"], False, transcripts[values["transcriptId"]][values["chromosome"]]["spanId"], values["frame"], '', values["source"]])

    spansTable.flush()
    spansTable.close()
    outputFile = open("result.txt", 'w')
    outputFile.write(spansTable.get_id())
    outputFile.close()
    print spansTable.get_id()
    return spansTable.get_id()

def writeEntry(spansTable, spanId, exonInfo, additionalColumns, chromosome, lo, hi, attributes, entry):
    if [lo, hi] not in exonInfo[chromosome] and [lo, hi-2] not in exonInfo[chromosome]:
        checkOverlap = trimOverlap(exonInfo[chromosome], lo, hi)
        if checkOverlap["hi"] - checkOverlap["lo"] > 0:
            entry[1] = checkOverlap["lo"]
            entry[2] = checkOverlap["hi"]
            if entry[6] == "-":
                entry[10] = int(entry[10]) + checkOverlap["hiChange"]%3
            else:
                entry[10] = int(entry[10]) + checkOverlap["loChange"]%3

            exonInfo[chromosome].append([lo, hi])
            spanId += 1
            for x in additionalColumns:
                if attributes.get(x) != None:
                    entry.append(attributes[x])
                else:
                    entry.append('')
            spansTable.add_rows([entry])
    return spanId

def trimOverlap(exons, lo, hi):
    loChange = 0
    hiChange = 0
    for x in exons:
        if lo >= x[0] and lo <= x[1]:
            loChange = x[1] - lo
            lo = x[1]
        if hi >= x[0] and hi <= x[1]:
            hiChange  = hi - x[0]
            hi = x[0]
    return {"lo": lo, "hi": hi, "loChange": loChange, "hiChange": hiChange}

def splitExons(transcriptInfo, chromosome, lo, hi, strand):
    upstream = "5' UTR"
    downstream = "3' UTR"
    if strand == "-":
        upstream = "3' UTR"
        downstream = "5' UTR"

    result = [["CDS", lo, hi, True]]
    if hi < transcriptInfo[chromosome]["codingLo"]:
        result[0][0] = upstream
    elif lo > transcriptInfo[chromosome]["codingHi"]:
        result[0][0] = downstream
    if lo < transcriptInfo[chromosome]["codingLo"]:
        result[0][1] = transcriptInfo[chromosome]["codingLo"]
        result.append([upstream, lo, transcriptInfo[chromosome]["codingLo"], False])
    if hi > transcriptInfo[chromosome]["codingHi"]:
        result[0][2] = transcriptInfo[chromosome]["codingHi"]
        result.append([downstream, transcriptInfo[chromosome]["codingHi"], hi, False])
    return result



def parseLine(line, capturedTypes):
    tabSplit = line.split("\t")
    if len(tabSplit) == 1:
            tabSplit = line.split(" ")
            if len(tabSplit) < 9:
                raise dxpy.AppError("One row did not have 9 entries, it had 1 instead. Offending line: " + line)
            tabSplit[8] = " ".join(tabSplit[8:])
            tabSplit = tabSplit[:9]
    chromosome = tabSplit[0]
    source = tabSplit[1]
    typ = tabSplit[2]
    if capturedTypes.get(typ) == None:
        message = 'Permitted types: ' + " ,".join(capturedTypes.keys())
        raise dxpy.AppError("One row had a type which is not in the list of permitted types. " + message + "\nOffending line: " + line + "\nOffending type: " + typ)

    try:
        score = float(tabSplit[5])
    except ValueError:
        if tabSplit[5] == "." or tabSplit[5] == '':
            score = dxpy.NULL
        else:
            raise dxpy.AppError("The score for one line could not be translated into a number and was not \".\"" + "\nOffending line: " + line + "\nOffending value: " + tabSplit[5])

    if tabSplit[6] != "+" and tabSplit[6] != "-" and tabSplit[6] != ".":
        raise dxpy.AppError("The strand indicated for an element was not \"+\", \"-\", or \".\"" + "\nOffending line: " + line + "\nOffending value: " + tabSplit[6])
    else:
        strand = tabSplit[6]

    try:
        lo = int(tabSplit[3])-1
    except ValueError:
        raise dxpy.AppError("One of the start values was could not be translated to an integer. " + "\nOffending line: " + line + "\nOffending value: " + tabSplit[3])

    try:
        hi = int(tabSplit[4])
    except ValueError:
        raise dxpy.AppError("One of the start values was could not be translated to an integer. " + "\nOffending line: " + line + "\nOffending value: " + tabSplit[4])



    try:
        frame = int(tabSplit[7])
        if frame > 2 or frame < 0:
            raise dxpy.AppError("The frame indicated for an element was not \".\", \"0\", \"1\", or \"2\"" + "\nOffending line: " + line + "\nOffending value: " + tabSplit[7])
    except ValueError:
        if tabSplit[7] == ".":
            frame = -1
        else:
            raise dxpy.AppError("The frame indicated for an element was not \".\", \"0\", \"1\", or \"2\"" + "\nOffending line: " + line + "\nOffending value: " + tabSplit[7])

    lineAttributes = {}

    ##Extract the attributes from the file
    entrySplit = tabSplit[8].split(";")
    result = []
    for x in entrySplit:
        keyValue = x.strip().split(" ")
        key = keyValue[0]
        if key != '':
            if len(key) < 100:
                lineAttributes[key.strip('"')] = keyValue[1].strip('"')

    geneId = lineAttributes["gene_id"]
    transcriptId = lineAttributes["transcript_id"]

    geneName = geneId
    if "gene_name" in lineAttributes:
        geneName = lineAttributes["gene_name"]
    transcriptName = transcriptId
    if "transcript_name" in lineAttributes:
        transcriptName = lineAttributes["transcript_name"]

    values = {"chromosome": chromosome, "lo": lo, "hi": hi, "geneName": geneName, "transcriptName": transcriptName, "source": source, "type": typ, "strand": strand, "score": score, "frame": frame, "geneId": geneId, "transcriptId": transcriptId, "attributes": lineAttributes}
    return values

def constructTable(inputFileName):
    inputFile = open(inputFileName, 'r')
    attributes = {"gene_id" : True, "transcript_id": True}
    for line in inputFile:
        if line[0] != "#":
            tabSplit = line.split("\t")
            if len(tabSplit) == 1:
                tabSplit = line.split(" ")
                if len(tabSplit) < 9:
                    raise dxpy.AppError("One row did not have 9 entries, it had 1 instead. Offending line: " + line)
                tabSplit[8] = " ".join(tabSplit[8:])
                tabSplit = tabSplit[:9]

            if len(tabSplit) != 9:
                raise dxpy.AppError("One row did not have 9 entries, it had " + str(len(tabSplit)) + " instead. Offending line: " + line)
            else:
                entrySplit = tabSplit[8].split(";")
                geneIdPresent = False
                transcriptIdPresent = False
                result = []
                for x in entrySplit:
                    keyValue = x.strip().split(" ")
                    key = keyValue[0]
                    if key == "gene_id":
                        geneIdPresent = True
                    elif key == "transcript_id":
                        transcriptIdPresent = True
                    attributes[key] = True
            if not geneIdPresent:
                raise dxpy.AppError("One row did not have a gene_id Offending line: " + line)
            if not transcriptIdPresent:
                raise dxpy.AppError("One row did not have a gene_id Offending line: " + line)


    #Construct table
    schema = [
            {"name": "chr", "type": "string"},
            {"name": "lo", "type": "uint32"},
            {"name": "hi", "type": "uint32"},
            {"name": "name", "type": "string"},
            {"name": "span_id", "type": "int32"},
            {"name": "type", "type": "string"},
            {"name": "strand", "type": "string"},
            {"name": "score", "type": "float"},
            {"name": "is_coding", "type": "boolean"},
            {"name": "parent_id", "type": "int32"},
            {"name": "frame", "type": "int16"},
            {"name": "description", "type": "string"},
            {"name": "source", "type": "string"},
            {"name": "gene_id", "type": "string"},
            {"name": "transcript_id", "type": "string"}]

    additionalColumns = ['gene_id', 'transcript_id']
    for k, v in attributes.iteritems():
        if k != '' and k != 'gene_id' and k != 'transcript_id' and len(k) < 100:
            schema.append({"name": k, "type": "string"})
            additionalColumns.append(k)

    indices = [dxpy.DXGTable.genomic_range_index("chr","lo","hi", 'gri'), 
               dxpy.DXGTable.lexicographic_index([
                  dxpy.DXGTable.lexicographic_index_column("name", True, False),
                  dxpy.DXGTable.lexicographic_index_column("chr"),
                  dxpy.DXGTable.lexicographic_index_column("lo"),
                  dxpy.DXGTable.lexicographic_index_column("hi"),
                  dxpy.DXGTable.lexicographic_index_column("type")], "search")]
    spansTable = dxpy.new_dxgtable(columns=schema, indices=indices)
    return spansTable, additionalColumns

def unpack(input):
    m = magic.Magic()

    # determine compression format
    try:
        file_type = m.from_file(input)
    except:
        raise dxpy.AppError("Unable to identify compression format")


    # if we find a tar file throw a program error telling the user to unpack it
    if file_type == 'application/x-tar':
        raise dxpy.AppError("App does not support tar files.  Please unpack.")

    # since we haven't returned, the file is compressed.  Determine what program to use to uncompress
    uncomp_util = None
    if file_type == 'XZ compressed data':
        uncomp_util = 'xzcat'
    elif file_type[:21] == 'bzip2 compressed data':
        uncomp_util = 'bzcat'
    elif file_type[:20] == 'gzip compressed data':
        uncomp_util = 'zcat'
    elif file_type == 'POSIX tar archive (GNU)' or 'tar' in file_type:
        raise dxpy.AppError("Found a tar archive.  Please untar your sequences before importing")
    else:
        # just return input filename since it's already uncompressed
        return input

    if uncomp_util != None:

        # bzcat does not support -t.  Use non streaming decompressors for testing input
        test_util = None
        if uncomp_util == 'xzcat':
            test_util = 'xz'
        elif uncomp_util == 'bzcat':
            test_util = 'bzip2'
        elif uncomp_util == 'zcat':
            test_util = 'gzip'

        try:
            subprocess.check_call(" ".join([test_util, "-t", input]), shell=True)
        except subprocess.CalledProcessError:
            raise dxpy.AppError("File failed integrity check by "+uncomp_util+".  Compressed file is corrupted.")

    # with that in hand, unzip file.  If we find a tar archive then exit with error.
    try:
        with subprocess.Popen([uncomp_util, input], stdout=subprocess.PIPE).stdout as pipe:
            line = pipe.next()
        uncomp_type = m.from_buffer(line)
    except:
        raise dxpy.AppError("Error detecting file format after decompression")

    if uncomp_type == 'POSIX tar archive (GNU)' or 'tar' in uncomp_type:
        raise dxpy.AppError("Found a tar archive after decompression.  Please untar your files before importing")
    #elif 'ASCII text' not in uncomp_type:
        #raise dxpy.AppError("After decompression found file type other than plain text")

    try:
        subprocess.check_call(" ".join([uncomp_util, "--stdout", input, ">", "uncompressed.gtf"]), shell=True)
        return "uncompressed.gtf"
    except subprocess.CalledProcessError:
        raise dxpy.AppError("Unable to open compressed input for reading")

def main(**args):
    return importGTF(**args)

if __name__ == '__main__':
    importGTF()



# Copyright (C) 2013 DNAnexus, Inc.
#
# This file is part of dx-toolkit (DNAnexus platform client libraries).
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may not
#   use this file except in compliance with the License. You may obtain a copy
#   of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

'''
Utilities shared by dxpy modules.
'''

import os, sys, collections, concurrent.futures, signal, traceback, time, gc
import dateutil.parser
from exec_utils import *


# Monkeypatch ThreadPoolExecutor with relevant logic from the patch for
# Python issue 16284. See:
#
#   <http://bugs.python.org/issue16284>
#   <http://hg.python.org/cpython/rev/70cef0a160cf/>
#
# We may need to apply the relevant parts of the patches to
# ProcessPoolExecutor and multiprocessing.Queue if we ever start using
# those, too.
def _non_leaky_worker(executor_reference, work_queue):
    try:
        while True:
            try:
                work_item = work_queue.get(block=True, timeout=0.1)
            except concurrent.futures.thread.queue.Empty:
                executor = executor_reference()
                # Exit if:
                #   - The interpreter is shutting down OR
                #   - The executor that owns the worker has been collected OR
                #   - The executor that owns the worker has been shutdown.
                if concurrent.futures.thread._shutdown or executor is None or executor._shutdown:
                    return
                del executor
            else:
                work_item.run()
                del work_item # <= free this item before the next
                              #    work_queue.get call runs, rather than
                              #    after
    except BaseException:
        concurrent.futures.thread._base.LOGGER.critical('Exception in worker', exc_info=True)

concurrent.futures.thread._worker = _non_leaky_worker


def _force_quit(signum, frame):
    # traceback.print_stack(frame)
    os._exit(os.EX_IOERR)
    # os.abort()

def get_futures_threadpool(max_workers):
    #if force_quit_on_sigint:
    #    signal.signal(signal.SIGINT, _force_quit)
    return concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)

def wait_for_a_future(futures, print_traceback=False):
    '''
    Return the next future that completes.  If a KeyboardInterrupt is
    received, then the entire process is exited immediately.  See
    wait_for_all_futures for more notes.
    '''
    while True:
        try:
            future = concurrent.futures.as_completed(futures, timeout=10000000000).next()
            break
        except concurrent.futures.TimeoutError:
            pass
        except KeyboardInterrupt as e:
            if print_traceback:
                traceback.print_stack()
            else:
                print ''
            os._exit(os.EX_IOERR)

    return future

def wait_for_all_futures(futures, print_traceback=False):
    '''
    Wait indefinitely for all futures in the input iterable to complete.
    Use a timeout to enable interrupt handling.
    Call os._exit() in case of KeyboardInterrupt. Otherwise, the atexit registered handler in concurrent.futures.thread
    will run, and issue blocking join() on all worker threads, requiring us to listen to events in worker threads
    in order to enable timely exit in response to Ctrl-C.

    Note: This still doesn't handle situations where Ctrl-C is pressed elsewhere in the code and there are worker
    threads with long-running tasks.

    Note: os._exit() doesn't work well with interactive mode (e.g. ipython). This may help:
    import __main__ as main; if hasattr(main, '__file__'): os._exit() else: os.exit()
    '''
    try:
        while True:
            waited_futures = concurrent.futures.wait(futures, timeout=60)
            if len(waited_futures.not_done) == 0:
                break
    except KeyboardInterrupt as e:
        if print_traceback:
            traceback.print_stack()
        else:
            print ''
        os._exit(os.EX_IOERR)

def response_iterator(request_iterator, worker_pool, max_active_tasks=4):
    '''
    :param request_iterator: This is expected to be an iterator producing inputs for consumption by the worker pool.
    :param worker_pool: Assumed to be a concurrent.futures.Executor instance.
    :max_active_tasks: The maximum number of tasks that may be either running or waiting for consumption of their result.

    Rate-limited asynchronous multithreaded task runner.
    Consumes tasks from *request_iterator*. Yields their results in order, while allowing up to *max_active_tasks* to run
    simultaneously. Unlike concurrent.futures.Executor.map, prevents new tasks from starting while there are
    *max_active_tasks* or more unconsumed results.
    '''

    # Debug fallback
    #for _callable, args, kwargs in request_iterator:
    #    yield _callable(*args, **kwargs)
    #return

    future_deque = collections.deque()
    for i in range(max_active_tasks):
        try:
            _callable, args, kwargs = request_iterator.next()
            # print "Submitting (initial batch):", _callable, args, kwargs
            f = worker_pool.submit(_callable, *args, **kwargs)
            future_deque.append(f)
        except StopIteration:
            break

    while len(future_deque) > 0:
        f = future_deque.popleft()
        if not f.done():
            wait_for_all_futures([f])
        if f.exception() is not None:
            raise f.exception()
        try:
            _callable, args, kwargs = request_iterator.next()
            # print "Submitting", _callable, args, kwargs
            next_future = worker_pool.submit(_callable, *args, **kwargs)
            future_deque.append(next_future)
        except StopIteration:
            pass
        yield f.result()
        del f # Free the future we just consumed now, instead of next
              # time around the loop
        gc.collect()

def string_buffer_length(buf):
    orig_pos = buf.tell()
    buf.seek(0, os.SEEK_END)
    buf_len = buf.tell()
    buf.seek(orig_pos)
    return buf_len

def normalize_time_input(t):
    ''' Converts inputs such as:
       "2012-05-01"
       "-5d"
       1352863174
    to milliseconds since epoch. See http://labix.org/python-dateutil and :meth:`normalize_timedelta`.
    '''
    error_msg = 'Error: Could not parse {t} as a timestamp or timedelta.  Expected a date format or an integer with a single-letter suffix: s=seconds, m=minutes, h=hours, d=days, w=weeks, M=months, y=years, e.g. "-10d" indicates 10 days ago'
    if isinstance(t, basestring):
        try:
            t = normalize_timedelta(t)
        except ValueError:
            try:
                t = int(time.mktime(dateutil.parser.parse(t).timetuple())*1000)
            except ValueError:
                raise ValueError(error_msg.format(t=t))
    if t < 0:
        t += int(time.time()*1000)
    return t

def normalize_timedelta(timedelta):
    '''
    Given a string like "1w" or "-5d", convert it to an integer in milliseconds.
    Note: not related to the datetime timedelta class.
    '''
    try:
        return int(timedelta)
    except ValueError:
        t, suffix = timedelta[:-1], timedelta[-1:]
        suffix_multipliers = {'s': 1000, 'm': 1000*60, 'h': 1000*60*60, 'd': 1000*60*60*24, 'w': 1000*60*60*24*7,
                              'M': 1000*60*60*24*30, 'y': 1000*60*60*24*365}
        if suffix not in suffix_multipliers:
            raise ValueError()
        return int(t) * suffix_multipliers[suffix]

# See http://stackoverflow.com/questions/4126348
class OrderedDefaultdict(collections.OrderedDict):
    def __init__(self, *args, **kwargs):
        newdefault = None
        newargs = ()
        if args:
            newdefault = args[0]
            if not (newdefault is None or callable(newdefault)):
                raise TypeError('first argument must be callable or None')
            newargs = args[1:]
        self.default_factory = newdefault
        super(self.__class__, self).__init__(*newargs, **kwargs)

    def __missing__ (self, key):
        if self.default_factory is None:
            raise KeyError(key)
        self[key] = value = self.default_factory()
        return value

    def __reduce__(self):
        args = self.default_factory if self.default_factory else tuple()
        return type(self), args, None, None, self.items()

def group_array_by_field(array, field='group'):
    groups = OrderedDefaultdict(list)
    for item in array:
        if field not in item and None not in groups:
            groups[None] = []
        groups[item.get(field)].append(item)
    return groups
